@@ -253,10 +253,6 @@ KConfigIniBackend::ParseInfo KConfigIniBackend::parseConfig(const QByteArray &cu
                 }
             }
 
-            if (!(entryOptions & KEntryMap::EntryRawKey)) {
-                printableToString(aKey, file, lineNo);
-            }
-
             if (options & ParseGlobal) {
                 entryOptions |= KEntryMap::EntryGlobal;
             }
@@ -292,7 +288,7 @@ KConfigIniBackend::ParseInfo KConfigIniBackend::parseConfig(const QByteArray &cu
     return fileOptionImmutable ? ParseImmutable : ParseOk;
 }
 
-void KConfigIniBackend::writeEntries(const QByteArray &locale, QIODevice &file, const KEntryMap &map, bool defaultGroup, bool &firstEntry)
+void KConfigIniBackend::writeEntries(const QByteArray &locale, QIODevice &file, const KEntryMap &map, bool defaultGroup, bool primaryGroup, bool &firstEntry)
 {
     QString currentGroup;
     bool groupIsImmutable = false;
@@ -301,6 +297,10 @@ void KConfigIniBackend::writeEntries(const QByteArray &locale, QIODevice &file,
         if ((key.mGroup != QStringLiteral("<default>")) == defaultGroup) {
             continue; // skip
         }
+        // Either process the primary group or all others
+        if ((mPrimaryGroup.isNull() || key.mGroup != mPrimaryGroup) == primaryGroup) {
+            continue; // skip
+        }
 
         // the only thing we care about groups is, is it immutable?
         if (key.mKey.isNull()) {
@@ -387,10 +387,15 @@ void KConfigIniBackend::writeEntries(const QByteArray &locale, QIODevice &file,
     bool firstEntry = true;
 
     // write default group
-    writeEntries(locale, file, map, true, firstEntry);
+    writeEntries(locale, file, map, true, false, firstEntry);
+
+    if (!mPrimaryGroup.isNull()) {
+        // write the primary group - it needs to be written before all other groups
+        writeEntries(locale, file, map, false, true, firstEntry);
+    }
 
     // write all other groups
-    writeEntries(locale, file, map, false, firstEntry);
+    writeEntries(locale, file, map, false, false, firstEntry);
 }
 
 bool KConfigIniBackend::writeConfig(const QByteArray &locale, KEntryMap &entryMap, WriteOptions options)
@@ -955,4 +960,9 @@ void KConfigIniBackend::setLocalFilePath(const QString &file)
     mLocalFilePath = file;
 }
 
+void KConfigIniBackend::setPrimaryGroup(const QString &group)
+{
+    mPrimaryGroup = group;
+}
+
 #include "moc_kconfigini_p.cpp"
